/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

/**
 * @file			ScatterWeb.ADDA.h
 * @ingroup	libADDA
 * @brief			Access functions for the AD/DA converter
 * 
 * @author		Bastian Blywis
 * @date			Mi Okt 31 11:59:35 2007
 * @since 		1.2
 * @version	$Revision$
 */

/**
 * @ingroup	libADDA
 * @{
 */

#ifndef __ADDA_H__
#define __ADDA_H__

/******************************************************************************
 * @name	 initialisation and usage
 * @{
 */

/**
 * @brief Battery voltage
 * Latest known voltage of VeRef (battery or lithium-ion-pack), updated every ADDA_VREF_S seconds
 */
float ADDA_VeRef;

/**
 * @brief Enables and starts DAC.
 * @return true if DAC was already enabled, else false
 */
bool ADDA_enableDAC();

/**
 * @brief Stops and disables DAC.
 * @return true if DAC was enabled prior disabling, else false
 */
bool ADDA_disableDAC();

/**
 * @brief Enables the ADC, uses ~1 mA
 * @return true if ADC was already enabled, else false
 */
bool ADDA_enableADC();

/**
 * @brief Disables the ADC
 * @return true if ADC was enabled prior disableling, else false
 */
bool ADDA_disableADC();

/**
 * @brief Measure voltage
 * @param adc out of {4,5}
 * @return U/[V], negative value on error
 * Measure voltage from ADC 4 or 5.
 */
float ADDA_readADC(uint8_t adc);

/**
 * @brief Set DAC output voltage.
 * The DAC value is calculated through successive approximation and may differ
 * from the requested value. The set voltage is returned.<br>
 * Please note: You can't output a voltage higher than ADDA_VeRef (might be initially 
 * 3.0V but may decrease during runtime)
 * @param dac out of {0,1}
 * @param volt out of [0.0 - 3.0]
 * @param approx set to 0 if output shall be volt but not bigger, set 1 if volt should be the minimal output
 * @return voltage set, -1 if volt not in [0.0 - 3.0], -2 if volt > ADDA_VeRef
 */
float ADDA_setDAC(uint8_t dac, float volt, uint8_t approx);

/**
 * @brief Measure temperature
 * Meassures and returns the current temperatur via the build in diode.<br>
 * The result may be unreliable due to the MSP generating heat itself. For more accurate results
 * use the SHT11 and the provided functions.
 * @return current temperature in [C]
 */
float ADDA_readTemperature();

#endif /*__ADDA_H__*/
/** @} */
