#include "fat.internals.h"

/**
 * @name Directory navigation
 */
void Fat_GetRootDir(const fat_state_t* pFS, file_handle_t* hRoot ) {
	hRoot->StartCA = pFS->RootDirCA;
	hRoot->Flags = FILE_HANDLE_FLAG_ROOTDIR;
	hRoot->Size = pFS->RootDirSz;
	hRoot->Size <<= pFS->BytsPerSec_bit;
	File_Reset(hRoot);	
}

bool Fat_GetNextDirEntry(
		const fat_state_t*	pFS, 
		file_handle_t*		hParent,
		uint32_t*				pAddress,
		struct fat_direntry*		resultDirEntry)
{
	uint8_t			ret;
	struct fat_direntry  dirEntry;
	uint8_t*			nameFlag;
	
	if( hParent->Flags & FILE_HANDLE_FLAG_EOF )
		return false;
	
	if( *pAddress == 0 ) {
		// first call
		*pAddress = _Fat_GetFilePos(pFS, hParent);	
	}
	nameFlag = (uint8_t*)&dirEntry;
	
	while(1) {
		// Test EOF
		if( hParent->Flags & FILE_HANDLE_FLAG_EOF )
			break;

		// read
		#if !SD_CACHE
			Sd_SetBlockLength(5); // 32
		#endif
		ret = Sd_read( &dirEntry, *pAddress, FAT_DIRENTRY_SIZE );
		RETF(ret);
	
		// advance		
		File_Seek( pFS, hParent, ret, pAddress );
	
		if( *nameFlag == FAT_NAME_FLAG_DELETED )
			continue;
		if( (dirEntry.Attributes & FAT_ATTR_LONG_NAME) == FAT_ATTR_LONG_NAME )
			continue;
		if( *nameFlag == FAT_NAME_FLAG_END )	
			break;
			
		// result
		if( resultDirEntry )
			memcpy( resultDirEntry, &dirEntry, sizeof(dirEntry) );

		return true;
	}
	
	hParent->Flags |= FILE_HANDLE_FLAG_EOF;
	return false;
}

void Fat_DirEntryToHandle(file_handle_t* hDir, uint32_t* pAddress, struct fat_direntry* dirEntry) {
	hDir->DirEntryPos = *pAddress;
	hDir->StartCA = dirEntry->FstClusLO;
	#if FAT32_SUPPORT
		memcpy((uint8_t*)&hDir->StartCA, dirEntry->FstClusHI, 2);
		//hDir->StartCA |= (dirEntry->FstClusHI << 16);
	#endif
	hDir->Size = dirEntry->FileSize;	
	hDir->Flags = FILE_HANDLE_FLAG_NONE;
	if( dirEntry->Attributes & FAT_ATTR_DIRECTORY )
		hDir->Flags |= FILE_HANDLE_FLAG_DIRECTORY;	
	File_Reset(hDir);
}



#if FAT_CHANGEDIR
	/**
	 * @param	hCurDir		Pointer to file handle structure which holds the current directory
	 * 						(initialize to Root directory before first call)
	 * @param	name		Pointer to character string of up to 11 chars with the name of
	 * 						the directory to which to change.
	 * 						@see Fat_FindFile for syntax of directory name
	 * 
	 * ChangeDir function mimicks the well know DOS shell CD command.
	 * The current directory pointer is modified every time, when the directory
	 * was successfully to change to has been found.
	 * 
	 * This function is a wrapper for Fat_FindFile adding dot and dotdot directory handling.
	 */
	bool Fat_ChangeDir(fat_state_t*	pFS, file_handle_t* hCurDir, char* name) {
		struct fat_direntry	dirEntry;
		
		File_Reset(hCurDir);
		if( name[0] == '.' ) {			
			if( name[1] == '.' ) {
				uint32_t	address;

				File_Seek( pFS, hCurDir, FAT_DIRENTRY_SIZE, NULL );				
				address = _Fat_GetFilePos(pFS, hCurDir);

				#if !SD_CACHE
					Sd_set_blocklength(5); // 32
				#endif
				Sd_read( &dirEntry, address, FAT_DIRENTRY_SIZE );
				Fat_DirEntryToHandle( hCurDir, &address, &dirEntry );
			}
			return true;
		}
		return Fat_FindFileSimple( pFS, hCurDir, name, FAT_ATTR_DIRECTORY, &dirEntry, hCurDir );
	}
#endif
///@}
