/*
Copyright 2007, Freie Universitaet Berlin. All rights reserved.

These sources were developed at the Freie Universitt Berlin, Computer
Systems and Telematics group.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are
met:

- Redistributions of source code must retain the above copyright
notice, this list of conditions and the following disclaimer.

- Redistributions in binary form must reproduce the above copyright
notice, this list of conditions and the following disclaimer in the
documentation and/or other materials provided with the distribution.
 
- Neither the name of Freie Universitaet Berlin (FUB) nor the names of its
contributors may be used to endorse or promote products derived from
this software without specific prior written permission.

This software is provided by FUB and the contributors on an "as is"
basis, without any representations or warranties of any kind, express
or implied including, but not limited to, representations or
warranties of non-infringement, merchantability or fitness for a
particular purpose. In no event shall FUB or contributors be liable
for any direct, indirect, incidental, special, exemplary, or
consequential damages (including, but not limited to, procurement of
substitute goods or services; loss of use, data, or profits; or
business interruption) however caused and on any theory of liability,
whether in contract, strict liability, or tort (including negligence
or otherwise) arising in any way out of the use of this software, even
if advised of the possibility of such damage.

This implementation was developed by the CST group at the FUB.

For documentation and questions please use the web site
http://scatterweb.mi.fu-berlin.de and the mailinglist
scatterweb@lists.spline.inf.fu-berlin.de (subscription via the Website).
Berlin, 2007
*/

/**
 * @file	ScatterWeb.SHT11.h
 * @ingroup	libSHT11
 * @brief	SHT11 library, Public interface
 * 
 * @author	Bastian Blywis	<blywis@inf.fu-berlin.de>
 * @date	Aug 2007
 * @since		1.1
 * @version	$Revision$
 */

/**
 * @ingroup	libSHT11
 * @{
 */

#ifndef __SHT11_H__
#define __SHT11_H__

/******************************************************************************
 * @name	 initialisation and usage
 * @{
 */

/**
 * @brief Conversion resolution selection
 *
 * Please note higher resolutions require longer conversion
 * times.The exact times may vary by +/- 15%.
 */
typedef enum {
	SHT11_high = 0x00,	///< 12 bit rel. Humidity (55ms) and 14 bit temperature (210ms)
	SHT11_low = 0x01		///< 8 bit rel. Humidity (11ms) and 12 bit temperature (55ms)
} SHT11_resolution;


/**
 * @brief Error codes
 *
 * These error codes are returned on errors instead of the temperature or
 * humidity. The values are way out of the supported range of the SHT11
 * and therefore easily noticed.
 */
typedef enum {
	SHT11_error = 1000,		///< general error, e.g. Sensibus read/write error, crc error, ...
	SHT11_busy = 2000,		///< the sensor is busy right now
	SHT11_missing = 3000	///< there is no SHT11 present
} SHT11_errorCodes;

/**
 * @brief Temperature unit selection
 *
 * Choose one of these unit types to get the current temperature
 * (via SHT11_temperature()) either in C or Fahrenheit.
 */
typedef enum {
	SHT11_celsius,				///< read temperature in C
	SHT11_fahrenheit		///< read temperature in F
} SHT11_tempUnit;

/**
 * @brief Select a conversion resolution
 * @see SHT11_resolution
 *
 * Set the converter to high or low resolution.
 */
void SHT11_setResolution(SHT11_resolution res);

/**
* @brief Initialize the SHT11
*
* Initialize various imporant ("private") members and the SHT11 by a hardware reset.
*/
void libSHT11_Init();

/**
 * @brief Read (uncorrected) humidity
 * @see SHT11_humidity
 * @return temperature in % relative humidity, 3000.0 on error
 *
 * Read the current values of the humidity sensor. This function returns
 * a non temperature coefficent corrected value and is therefore faster.
 */
float SHT11_humidityUncorrected();

/**
 * @brief Read (corrected) humidity
 * @see SHT11_humidityUncorrected
 * @return temperature in % relative humidity
 *
 * Read the current values of the humidity sensor.
 * This function takes more time to meassure the humidity
 * because the temperature is needed to calculate a correction
 * factor.
 */
float SHT11_humidity();

/**
 * @brief Read temperature
 * @return temperature in C or F
 *
 * Read the value of the temperature sensor.
 */
float SHT11_temperature(SHT11_tempUnit unit);


#endif /*__SHT11_H__*/
/** @} */
